//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <azac_api_cxx_details_json.h>
#include <vision_api_cxx_common.h>
#include <vision_api_cxx_details_result_base.h>
#include <vision_api_cxx_image_analysis_result_properties.h>
#include <vision_api_cxx_image_analysis_result_property.h>
#include <vision_api_cxx_image_analysis_result_reason.h>
#include <vision_api_cxx_image_analysis_core_result_reason.h>
#include <vision_api_cxx_image_analysis_core_stop_reason.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace ImageAnalysis {

using namespace Azure::AI::Core::Details::Json;
using namespace Azure::AI::Vision::Core::Details;
using namespace Azure::AI::Vision::ImageAnalysis::Details;

/// <summary>
/// Represents a rectangular area inside an image.
/// </summary>
class Rectangle
{
public:
    /// <summary> 
    /// X-coordinate of the top left point of the area, in pixels.
    /// </summary>
    int32_t X = 0;

    /// <summary> 
    /// Y-coordinate of the top left point of the area, in pixels.
    /// </summary>
    int32_t Y = 0;

    /// <summary> 
    /// Height measured from the top-left point of the area, in pixels.
    /// </summary>
    int32_t H = 0;

    /// <summary>
    /// Width measured from the top-left point of the area, in pixels.
    /// </summary>
    int32_t W = 0;

    /// <summary>
    /// Helper method to display the Rectangle values
    /// </summary>
    /// <returns>
    /// A string containing the values of X, Y, H and W.
    /// </returns>
    std::string ToString() const
    {
        return "{X=" + std::to_string(X) +
               ",Y=" + std::to_string(Y) +
               ",Width=" + std::to_string(W) +
               ",Height=" + std::to_string(H) + "}";
    }

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the Rectangle object from the appropriate JSON element in the service response.
    /// </summary>
    Rectangle(JsonValue jsonBoundingBox)
    {
        X = (int32_t)(jsonBoundingBox["x"].AsInt());
        Y = (int32_t)(jsonBoundingBox["y"].AsInt());
        W = (int32_t)(jsonBoundingBox["w"].AsInt());
        H = (int32_t)(jsonBoundingBox["h"].AsInt());
    }

    Rectangle() = default;

    /*! \endcond */
};

/// <summary>
/// Represents a physical object detected in an image.
/// </summary>
class DetectedObject
{
public:
    /// <summary>
    /// A label that briefly describes the detected object.
    /// </summary>
    std::string Name;

    /// <summary>
    /// A score that represents the likelihood that this detection was accurate.
    /// </summary>
    /// <remarks>
    /// Confidence scores span the range of 0.0 to 1.0 (inclusive), with higher values indicating higher
    /// probability.
    /// </remarks>
    double Confidence = 0;

    /// <summary>
    /// A rectangular boundary within which the object was detected.
    /// </summary>
    /// <remarks>
    /// Coordinates are are in pixels, with (0,0) being the top-left of the source image.
    /// </remarks>
    Rectangle BoundingBox;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedObject from the appropriate JSON element in the service response.
    /// </summary>
    DetectedObject(JsonValue jsonObjectResult)
    {
        Name =  jsonObjectResult["tags"][0]["name"].AsString();
        Confidence =  jsonObjectResult["tags"][0]["confidence"].AsDouble();
        BoundingBox = Rectangle(jsonObjectResult["boundingBox"]);
    }

    /*! \endcond */
};

/// <summary>
/// Represents a list of physical object detected in an image.
/// </summary>
class DetectedObjects: public std::vector<DetectedObject>
{
public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedObjects from the appropriate JSON element in the service response.
    /// </summary>
    DetectedObjects(JsonValue jsonObjectsResult)
    {
        if (jsonObjectsResult.HasValue("values"))
        {
            JsonValue jsonObjectsResultValues = jsonObjectsResult["values"];

            for (int i = 0; i < jsonObjectsResultValues.Count(); ++i)
            {
                push_back(DetectedObject(jsonObjectsResultValues[i]));
            }
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represent an image tag. A tag can be a recognizable object, living being, scenery, or actions that appear in the image.
/// </summary>
class ContentTag
{
public:
    /// <summary>
    /// Name of the tag.
    /// </summary>
    /// <remarks>
    /// Note that you can control the tag language by calling ImageAnalysisOptions::SetLanguage().
    /// </remarks>
    std::string Name;

    /// <summary>
    /// A score that represents the likelihood that this detection was accurate.
    /// </summary>
    /// <remarks>
    /// Confidence scores span the range of 0.0 to 1.0 (inclusive), with higher values indicating higher
    /// probability.
    /// </remarks>
    double Confidence = 0;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the ContentTag object from the appropriate JSON element in the service response.
    /// </summary>
    ContentTag(JsonValue jsonTagsResultValue)
    {
        Name = jsonTagsResultValue["name"].AsString();
        Confidence = jsonTagsResultValue["confidence"].AsDouble();
    }

    /*! \endcond */
};

/// <summary>
/// Represents a list of image tags
/// </summary>
class ContentTags: public std::vector<ContentTag>
{
public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the ContentTags object from the appropriate JSON element in the service response.
    /// </summary>
    ContentTags(JsonValue jsonTagsResult)
    {
        if (jsonTagsResult.HasValue("values"))
        {
            JsonValue jsonTagsResultValues = jsonTagsResult["values"];

            for (int i = 0; i < jsonTagsResultValues.Count(); ++i)
            {
                push_back(ContentTag(jsonTagsResultValues[i]));
            }
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represents a person detected in an image.
/// </summary>
class DetectedPerson
{
public:
    /// <summary>
    /// A score that represents the likelihood that this detection was accurate.
    /// </summary>
    /// <remarks>
    /// Confidence scores span the range of 0.0 to 1.0 (inclusive), with higher values indicating higher
    /// probability.
    /// </remarks>
    double Confidence = 0;

    /// <summary>
    /// A rectangular boundary within which the person was detected.
    /// </summary>
    /// <remarks>
    /// Coordinates are are in pixels, with (0,0) being the top-left of the source image.
    /// </remarks>
    Rectangle BoundingBox;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedPerson object from the appropriate JSON element in the service response.
    /// </summary>
    DetectedPerson(JsonValue jsonPeopleResultsValue)
    {
        Confidence = jsonPeopleResultsValue["confidence"].AsDouble();
        BoundingBox = Rectangle(jsonPeopleResultsValue["boundingBox"]);
    }

    /*! \endcond */
};

/// <summary>
/// Represents people detected in an image.
/// </summary>
class DetectedPeople: public std::vector<DetectedPerson>
{
public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedPeople object from the appropriate JSON element in the service response.
    /// </summary>
    DetectedPeople(JsonValue jsonPeopleResult)
    {
        if (jsonPeopleResult.HasValue("values"))
        {
            JsonValue jsonPeopleResultsValues = jsonPeopleResult["values"];

            for (int i = 0; i < jsonPeopleResultsValues.Count(); ++i)
            {
                push_back(DetectedPerson(jsonPeopleResultsValues[i]));
            }
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represents a suggested image cropping that preserves much of the image content.
/// </summary>
class CropSuggestion
{
public:
    /// <summary>
    /// The aspect ratio of this crop suggestion
    /// </summary>
    /// <remarks>
    /// Aspect ratios are calculated by dividing the width of the cropped region by its height.
    ///
    /// You can request particular aspect ratios by calling the method ImageAnalysisOptions::SetCroppingAspectRatios.
    ///
    /// AspectRatio will be in the range 0.75 to 1.8 (inclusive) if ImageAnalysisOptions::SetCroppingAspectRatios was called,
    /// otherwise it will be in the range 0.5 to 2.0 (inclusive).
    /// </remarks>
    double AspectRatio = 0;

    /// <summary>
    /// Gets the rectangular bounds of the crop suggestion.
    /// </summary>
    /// <remarks>
    /// Coordinates are are in pixels, with (0,0) being the top-left of the source image.
    /// </remarks>
    Rectangle BoundingBox;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the CropSuggestion object from the appropriate JSON element in the service response.
    /// </summary>
    CropSuggestion(JsonValue jsonSmartCropsResultValue)
    {
        AspectRatio = jsonSmartCropsResultValue["aspectRatio"].AsDouble();
        BoundingBox = Rectangle(jsonSmartCropsResultValue["boundingBox"]);
    }

    /*! \endcond */
};

/// <summary>
/// Represents a list of image crop suggestions that preserve most of the image content
/// </summary>
class CropSuggestions: public std::vector<CropSuggestion>
{
public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the CropSuggestions object from the appropriate JSON element in the service response.
    /// </summary>
    CropSuggestions(JsonValue jsonSmartCropsResult)
    {
        if (jsonSmartCropsResult.HasValue("values"))
        {
            JsonValue jsonSmartCropsResultValues = jsonSmartCropsResult["values"];

            for (int i = 0; i < jsonSmartCropsResultValues.Count(); ++i)
            {
                push_back(CropSuggestion(jsonSmartCropsResultValues[i]));
            }
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represents a generated phrase that describes the content of the image.
/// </summary>
class ContentCaption
{
public:
    /// <summary>
    /// A generated phrase that describes the content of the image.
    /// </summary>
    std::string Content;

    /// <summary>
    /// A score, in the range of 0 to 1, representing the confidence that this description is accurate.
    /// </summary>
    /// <remarks>
    /// Confidence scores span the range of 0.0 to 1.0 (inclusive), with higher values indicating higher
    /// probability.
    /// </remarks>
    double Confidence = 0;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the ContentCaption object from the appropriate JSON element in the service response.
    /// </summary>
    ContentCaption(JsonValue jsonCaptionResult)
    {
        Content = jsonCaptionResult["text"].AsString();
        Confidence = jsonCaptionResult["confidence"].AsDouble();
    }

    /*! \endcond */
};

/// <summary>
/// Represents a single word that was detected in an image.
/// </summary>
/// <remarks>
/// Words consist of a contiguous sequence of characters.
/// For non-space delimited languages such as Chinese, Japanese, and Korean, each character is represented as its
/// own word.
/// </remarks>
class DetectedTextWord
{
public:
    /// <summary>
    /// The text detected within the bounds of this word.
    /// </summary>
    std::string Content;

    /// <summary>
    /// A score that represents the likelihood that this detection was accurate.
    /// </summary>
    /// <remarks>
    /// Confidence scores span the range of 0.0 to 1.0 (inclusive), with higher values indicating higher
    /// probability.
    /// </remarks>
    double Confidence = 0;

    /// <summary>
    /// A bounding polygon with points that enclose the word.
    /// </summary>
    /// <remarks>
    /// These points are polygon vertices, presented in clockwise order from the left (-180 degrees, inclusive)
    /// relative to the region's orientation.
    ///
    /// Coordinates are are in pixels, with (0,0) being the top-left of the source image.
    /// </remarks>
    std::vector<int32_t> BoundingPolygon;

private:
    /// <summary>
    /// Recorded to properly bound the substring lookup into the parent result's text.
    /// </summary>
    int32_t Offset = 0;

    /// <summary>
    /// Recorded to properly bound the substring lookup into the parent result's text.
    /// </summary>
    int32_t Length = 0;

public:
    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedTextWord object from the appropriate JSON element in the service response.
    /// </summary>
    DetectedTextWord(JsonValue jsonWordResult)
    {
        Content = jsonWordResult["content"].AsString();
        Confidence = jsonWordResult["confidence"].AsDouble();
        Offset = (int32_t)jsonWordResult["span"]["offset"].AsInt();
        Length = (int32_t)jsonWordResult["span"]["length"].AsInt();

        for (int j = 0; j < jsonWordResult["boundingBox"].Count(); ++j)
        {
            int32_t pixel = (int32_t)jsonWordResult["boundingBox"][j].AsInt();
            BoundingPolygon.push_back(pixel);
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represents a single, contiguous line of text as detected within an image.
/// </summary>
class DetectedTextLine
{
public:
    /// <summary>
    /// The text detected in this line.
    /// </summary>
    std::string Content;

    /// <summary>
    /// A bounding polygon with points that enclose this line of text.
    /// </summary>
    /// <remarks>
    /// These points are polygon vertices, presented in clockwise order from the left (-180 degrees, inclusive)
    /// relative to the region's orientation.
    ///
    /// Coordinates are are in pixels, with (0,0) being the top-left of the source image.
    /// </remarks>
    std::vector<int32_t> BoundingPolygon;

    /// <summary>
    /// A list of detected words associated with this line.
    /// </summary>
    std::vector<DetectedTextWord> Words;

    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedTextLine object from the appropriate JSON element in the service response.
    /// </summary>
    DetectedTextLine(JsonValue jsonLineResult, JsonValue jsonWordsResult)
    {
        Content = jsonLineResult["content"].AsString();

        // At the moment we only have a single element in the "spans" array
        int32_t lineOffset = (int32_t)jsonLineResult["spans"][0]["offset"].AsInt();
        int32_t lineLength = (int32_t)jsonLineResult["spans"][0]["length"].AsInt();

        for (int i = 0; i < jsonLineResult["boundingBox"].Count(); ++i)
        {
            int32_t pixel = (int32_t)jsonLineResult["boundingBox"][i].AsInt();
            BoundingPolygon.push_back(pixel);
        }

        // Find all the words that belong to this line based on the offset and length
        for (int i = 0; i < jsonWordsResult.Count(); ++i)
        {
            int32_t wordOffset = (int32_t)jsonWordsResult[i]["span"]["offset"].AsInt();
            int32_t wordLength = (int32_t)jsonWordsResult[i]["span"]["length"].AsInt();

            if (wordOffset >= lineOffset && wordOffset + wordLength <= lineOffset + lineLength)
            {
                Words.push_back(DetectedTextWord(jsonWordsResult[i]));
            }
        }
    }

    /*! \endcond */
};

/// <summary>
/// Represents the text lines detected in an image.
/// </summary>
class DetectedText
{
public:
    /// <summary>
    /// The full list of all lines of text detected in an image.
    /// </summary>
    std::vector<DetectedTextLine> Lines;

    /*! \cond PRIVATE */

    /// <summary>
    /// Construct the DetectedTextBlocks object from the appropriate JSON element in the service response.
    /// </summary>
    DetectedText(JsonValue jsonReadResultPage0)
    {
        for (int i = 0; i < jsonReadResultPage0["lines"].Count(); ++i)
        {
            Lines.push_back(DetectedTextLine(jsonReadResultPage0["lines"][i], jsonReadResultPage0["words"]));
        }
    }

    DetectedText() = default;

    /*! \endcond */
};

/// <summary>
/// Represents the outcome of an Image Analysis operation.
/// </summary>
/// <remarks>
/// When an operation was successful, this result includes analyzed data for the features specified
/// when calling ImageAnalysisOptions::SetFeatures.
///
/// Result data is selectively populated based on those features and will be empty if
/// not requested.

/// We use the Nullable class to hold a result, as a convenient container that can
/// indicate if a result has value (see Nullable::HasValue).
/// </remarks>
class ImageAnalysisResult : public Core::Details::ResultBase<ImageAnalysisCoreResultReason, ImageAnalysisResultProperty, ImageAnalysisResultProperties>
{
private:

    using BaseResult = ResultBase<ImageAnalysisCoreResultReason, ImageAnalysisResultProperty, ImageAnalysisResultProperties>;

public:

    /// <summary>
    /// Destructs an instance of the ImageAnalysisResult class.
    /// </summary>
    ~ImageAnalysisResult() = default;

    /// <summary>
    /// Gets a unique identifier from the Vision Service, associated with this result.
    /// </summary>
    /// <returns>
    /// The result ID string.
    /// </returns>
    std::string GetResultId() const { return BaseResult::GetResultId(); }

    /// <summary>
    /// Gets a unique identifier from the Vision Service, associated with this result.
    /// </summary>
    /// <returns>
    /// The unique ID string.
    /// </returns>
    template<class T = std::string> 
    AI::Core::Details::enable_if_w_or_string_t<T> GetResultId() const { return BaseResult::GetResultId<T>(); }

    /// <summary>
    /// Gets a value indicating why this result was generated.
    /// </summary>
    /// <returns>
    /// Reason of this result.
    /// </returns>
    ImageAnalysisResultReason GetReason() const
    { 
        return reason;
    }

    /// <summary>
    /// Gets a collection of additional properties on the result.
    /// </summary>
    const ImageAnalysisResultProperties& Properties;

    /// <summary>
    /// Gets the JSON response payload from the Vision Service that was deserialized
    /// to create the provided ImageAnalysisResult.
    /// </summary>
    /// <returns>
    /// A JSON string.
    /// </returns>
    std::string GetJsonResult()
    { 
        return Properties.Get(ImageAnalysisResultProperty::Json);
    }

    /// <summary>
    /// Gets the identifier of the analyzed image. This could be the full-path
    /// image file name or the image URL, depending on how the VisionSource was created.
    /// </summary>
    /// <returns>
    /// The image ID string.
    /// </returns>
    std::string GetImageId() const
    { 
        return Properties.Get("result.image.id");
    }

    /// <summary>
    /// Gets the model version used by the Image Analysis Service to create this result.
    /// </summary>
    /// <returns>
    /// The version of the model.
    /// </returns>
    Nullable<std::string> GetModelVersion()
    { 
        ParseJson();
        return modelVersion;
    }

    /// <summary>
    /// Gets the height, in pixels, of the analyzed image.
    /// </summary>
    /// <returns>
    /// A Nullable object holding an int32_t value representing the height, or an empty value
    /// </returns>
    /// <remarks>
    /// Only populated if the image was analyzed successfully
    /// </remarks>
    Nullable<int32_t> GetImageHeight()
    {
        ParseJson();
        return imageHeight;
    }

    /// <summary>
    /// Gets the width, in pixels, of the analyzed image. Only populated
    /// if the image was analyzed successfully.
    /// </summary>
    /// <returns>
    /// A Nullable object holding an int32_t value representing the width, or an empty value
    /// </returns>
    Nullable<int32_t> GetImageWidth()
    {
        ParseJson();
        return imageWidth;
    }

    /// <summary>
    /// Gets a generated phrase that describes the content of the analyzed image.
    /// </summary>
    /// <remarks>
    /// This result will only be populated if ImageAnalysisFeature::Caption was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the ContentCaption result or an empty value
    /// </returns>
    Nullable<ContentCaption> GetCaption()
    {
        ParseJson();
        return contentCaption;
    }

    /// <summary>
    /// Gets a list of detected objects from the analyzed image.
    /// </summary>
    /// <remarks>
    /// This result will only be populated if ImageAnalysisFeature::Objects was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the DetectedObjects result or an empty value
    /// <returns>
    Nullable<DetectedObjects> GetObjects()
    {
        ParseJson();
        return detectedObjects;
    }

    /// <summary>
    /// Gets a collection of extracted textual lines and words from an analyzed image.
    /// </summary>
    /// <remarks>
    /// This result will only be populated if ImageAnalysisFeature::Text was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the DetectedText result or an empty value.
    /// </returns>
    Nullable<DetectedText> GetText()
    {
        ParseJson();
        return detectedText;
    }

    /// <summary>
    /// Gets a list of content tag detections from the analyzed image.
    /// </summary>
    /// <remarks>
    /// This result will only be populated if ImageAnalysisFeature::Tags was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the ContentTags result or an empty value.
    /// </returns>
    Nullable<ContentTags> GetTags()
    {
        ParseJson();
        return contentTags;
    }

    /// <summary>
    /// Gets a list of detected people from an analyzed image.
    /// </summary>
    /// <remarks>
    /// This result will only be populated if ImageAnalysisFeature::People was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the DetectedPeople result or an empty value.
    /// </returns>
    Nullable<DetectedPeople> GetPeople()
    {
        ParseJson();
        return detectedPeople;
    }

    /// <summary>
    /// Gets a list of suggested image crop operations of the analyzed
    /// image at the desired aspect ratios (if provided)
    /// </summary>
    /// <remarks>
    /// Also knows as SmartCrops. These cropping suggestions preserve as much content 
    /// as possible while achieving the specified aspect ratios (if provided).
    ///
    /// This result will only be populated if ImageAnalysisFeature::CropSuggestions was included
    /// while calling ImageAnalysisOptions::SetFeatures.
    ///
    /// Optionally, specify one or more desired cropping aspect ratios by calling 
    /// ImageAnalysisOptions::SetCroppingAspectRatios. If CroppingAspectRatios is not set,
    /// the Service will return one crop suggestion with an aspect ratio it sees fit.
    /// </remarks>
    /// <returns>
    /// A Nullable object holding the CropSuggestions result or an empty value.
    /// </returns>
    Nullable<CropSuggestions> GetCropSuggestions()
    {
        ParseJson();
        return cropSuggestions;
    }

protected:
     /*! \cond PROTECTED */
    static std::shared_ptr<ImageAnalysisResult> FromHandle(AZAC_HANDLE handle)
    {
        auto ptr = new ImageAnalysisResult(handle);
        return std::shared_ptr<ImageAnalysisResult>(ptr);
    }

    explicit ImageAnalysisResult(AZAC_HANDLE result) :
        ResultBase(result),
        Properties(GetProperties())
    {
        reason = GetCombinedReason();
    }

    explicit operator AZAC_HANDLE() { return AI::Core::Details::ProtectedAccess<BaseResult>::HandleFromPtr(this); }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(ImageAnalysisResult);

    // True of the JSON result was already parsed
    bool jsonParsed = false;

    // Values parsed from the JSON response, if jsonParsed = true.
    // These values will be present in every HTTP 200 OK response.
    // The Nullable object will be empty if the service returned an error.
    Nullable<std::string> modelVersion;
    Nullable<int32_t> imageHeight;
    Nullable<int32_t> imageWidth;

    // Values parsed from the JSON response, if jsonParsed = true.
    // These are optional, depending on the visual features settings. Hence we use Nullable<>
    // values. The Nullable object will be empty if the value did not exist in the JSON response,
    // or if the service returned an error.
    Nullable<ContentTags> contentTags;
    Nullable<ContentCaption> contentCaption;
    Nullable<DetectedObjects> detectedObjects;
    Nullable<DetectedText> detectedText;
    Nullable<DetectedPeople> detectedPeople;
    Nullable<CropSuggestions> cropSuggestions;

    // The reason returned on this public result object, which may be different than ImageAnalysisCoreResultReason
    ImageAnalysisResultReason reason;

    void ParseJson()
    {
        if (jsonParsed)
        {
            return;
        }

        std::string insightsJson = Properties.Get(ImageAnalysisResultProperty::Json);

        if (reason == ImageAnalysisResultReason::Analyzed && !insightsJson.empty())
        {
            JsonValue jsonValueResults = JsonValue::Parse(insightsJson.c_str(), insightsJson.size());

            if (jsonValueResults.HasValue("modelVersion"))
            {
                modelVersion = jsonValueResults["modelVersion"].AsString();
            }

            if (jsonValueResults.HasValue("metadata"))
            {
                imageHeight = (int32_t)(jsonValueResults["metadata"]["height"].AsInt());
                imageWidth = (int32_t)(jsonValueResults["metadata"]["width"].AsInt());
            }

            if (jsonValueResults.HasValue("tagsResult"))
            {
                contentTags = ContentTags(jsonValueResults["tagsResult"]);
            }

            if (jsonValueResults.HasValue("objectsResult"))
            {
                detectedObjects = DetectedObjects(jsonValueResults["objectsResult"]);
            }

            if (jsonValueResults.HasValue("captionResult"))
            {
                contentCaption = ContentCaption(jsonValueResults["captionResult"]);
            }

            if (jsonValueResults.HasValue("readResult"))
            {
                detectedText = DetectedText(jsonValueResults["readResult"]["pages"][0]);
            }

            if (jsonValueResults.HasValue("peopleResult"))
            {
                detectedPeople = DetectedPeople(jsonValueResults["peopleResult"]);
            }

            if (jsonValueResults.HasValue("smartCropsResult"))
            {
                cropSuggestions = CropSuggestions(jsonValueResults["smartCropsResult"]);
            }
        }

        jsonParsed = true;
    }

    // Private helper method. Converts the result reason and (if applicable) stop reason on a result into the
    // combined, public representation of a result reason. We don't separate stop and result reasons in the public
    // surface because IA has no concept of 'stop' yet.
    ImageAnalysisResultReason GetCombinedReason()
    {
        const ImageAnalysisResultProperties& properties = BaseResult::GetProperties();

        // Error check #1: if an "error.reason" exists -- well, it's an error
        if (!properties.Get("error.reason", "").empty())
        {
            return ImageAnalysisResultReason::Error;
        }

        // Error check #2 -- if "session.stopped.reason" exists and it converts to "error," it's an error
        if (properties.Get("session.stopped.reason", "") == std::to_string(static_cast<int32_t>(Azure::AI::Vision::ImageAnalysis::Details::ImageAnalysisCoreStopReason::Error)))
        {
            return ImageAnalysisResultReason::Error;
        }

        // If it's not an error, the "result.reason" should indicate it's a successful analysis result
        if (properties.Get("result.reason", "") == std::to_string(static_cast<int32_t>(Azure::AI::Vision::ImageAnalysis::Details::ImageAnalysisCoreResultReason::Analyzed)))
        {
            return ImageAnalysisResultReason::Analyzed;
        }

        // If we got this far and none of the above fit? Something went wrong
        return ImageAnalysisResultReason::Error;
    }

    // Do not expose these public methods of the base class until Image Analysis supports analyzing a sequence of images in a video
    uint64_t GetAssociatedMediaPosition() const { return BaseResult::GetAssociatedMediaPosition(); }
    std::shared_ptr<Input::Frames::FrameReader> GetAssociatedFrameReader() { return nullptr; }
    std::shared_ptr<Input::Frames::FrameSetReader> GetAssociatedFrameSetReader() { return nullptr; }
};

} } } } // Azure::AI::Vision::ImageAnalysis
