//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once

#include <future>
#include <vision_api_cxx_core.h>
#include <vision_api_cxx_image_analysis_options.h>
#include <vision_api_cxx_image_analysis_result.h>
#include <vision_api_cxx_image_analysis_event_args.h>
#include <vision_api_cxx_image_analyzer_properties.h>
#include <vision_api_cxx_image_analyzer_property.h>
#include <vision_api_cxx_image_analysis_result_properties.h>
#include <vision_api_cxx_image_analysis_result_property.h>
#include <vision_api_cxx_image_analysis_result_reason.h>
#include <vision_api_cxx_image_analysis_error_details.h>
#include <vision_api_cxx_image_analysis_error_reason.h>
#include <vision_api_cxx_image_analysis_core_stop_reason.h>
#include <vision_api_cxx_image_analysis_core_result_reason.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace ImageAnalysis {

/// <summary>
/// An object that facilitates Image Analysis operations with the Computer Vision service
/// </summary>
/// <remarks>Use ImageAnalyzer::Create(service, input) to instantiate</remarks>
class ImageAnalyzer :
    public std::enable_shared_from_this<ImageAnalyzer>,
    private Core::Details::RecognizerBase<
        ImageAnalyzer,
        ImageAnalysisOptions,
        ImageAnalysisOption,
        ImageAnalysisOptions,
        ImageAnalysisOption,
        ImageAnalyzerProperties,
        ImageAnalyzerProperty,
        ImageAnalysisResult,
        ImageAnalysisEventArgs,
        ImageAnalysisResult,
        ImageAnalysisEventArgs,
        ImageAnalysisEventArgs,
        ImageAnalysisEventArgs>
{
protected:

    friend class Core::Details::RecognizerBase<ImageAnalyzer,
        ImageAnalysisOptions, ImageAnalysisOption,
        ImageAnalysisOptions, ImageAnalysisOption,
        ImageAnalysis::ImageAnalyzerProperties, ImageAnalysis::ImageAnalyzerProperty,
        ImageAnalysisResult, ImageAnalysisEventArgs,
        ImageAnalysisResult, ImageAnalysisEventArgs,
        ImageAnalysisEventArgs, ImageAnalysisEventArgs>;

    using BaseRecognizer = Core::Details::RecognizerBase<ImageAnalyzer,
        ImageAnalysisOptions, ImageAnalysisOption,
        ImageAnalysisOptions, ImageAnalysisOption,
        ImageAnalysis::ImageAnalyzerProperties, ImageAnalyzerProperty,
        ImageAnalysisResult, ImageAnalysisEventArgs,
        ImageAnalysisResult, ImageAnalysisEventArgs,
        ImageAnalysisEventArgs, ImageAnalysisEventArgs>;

public:

    /// <summary>
    /// Creates a new ImageAnalyzer instance
    /// </summary>
    /// <param name="serviceOptions">The Vision Service Options used to connect to the service</param>
    /// <param name="visionSource">The Vision Source to use</param>
    /// <param name="analysisOptions">The Image Analysis Options to use</param>
    /// <returns>The newly created ImageAnalyzer wrapped inside a std::shared_ptr</returns>
    static std::shared_ptr<ImageAnalyzer> Create(
        const std::shared_ptr<Service::VisionServiceOptions>& serviceOptions,
        const std::shared_ptr<Input::VisionSource>& visionSource,
        const std::shared_ptr<ImageAnalysisOptions>& analysisOptions)
    {
        const std::string imageAnalyzerViewKind = "image.analyzer";
        std::string viewKind = imageAnalyzerViewKind;

        if (analysisOptions)
        {
            std::string viewKindOverride = analysisOptions->Advanced.Get("view.kind", "");
            if (!viewKindOverride.empty())
            {
                viewKind = viewKindOverride;
            }
        }

        if (visionSource)
        {
            if (viewKind == imageAnalyzerViewKind)
            {
                // TODO: The input configurations below should be set by the REA instead of the binding layer.
                // Work item: https://msasg.visualstudio.com/Skyman/_workitems/edit/4257450

                // Configure the vision source used by ImageAnalyzer to provide the image file as-is, without parsing and decoding it.
                visionSource->Properties.Set("adapter.passthrough", true);

                // The service limits the size of the image it can process
                visionSource->Properties.Set("source.file.max.size", (size_t)(20 * 1024 * 1024));

                // Treat all input as image files (regardless of file name extension)
                visionSource->Properties.Set("source.file.type", 2 << 8 /* = Media::SourceType::ImageFile in source\core\types\include\enums\media.h */);
            }
        }

        return BaseRecognizer::Create(viewKind.c_str(), serviceOptions, visionSource, analysisOptions);
    }

    /// <summary>
    /// Destructs an instance of the ImageAnalyzer class.
    /// </summary>
    virtual ~ImageAnalyzer() { }

    /// <summary>
    /// Performs a single Image Analysis operation using the source provided when this ImageAnalyzer was created.
    /// This is a synchronous (blocking) call, that will return after the results are available from the service.
    /// </summary>
    /// <returns>The newly created ImageAnalysisResult wrapped inside a std::shared_ptr</returns>
    std::shared_ptr<ImageAnalysisResult> Analyze() { return BaseRecognizer::RecognizeOnce(); }

    /// <summary>
    /// Begins a single Image Analysis operation against the source provided when this ImageAnalyzer was created.
    /// </summary>
    /// <returns>The future ImageAnalysisResult wrapped inside a std::future<std::shared_ptr<>></returns>
    std::future<std::shared_ptr<ImageAnalysisResult>> AnalyzeAsync() { return BaseRecognizer::RecognizeOnceAsync(); }

    /// <summary>
    /// An event signal that is raised when a new ImageAnalysisResult is available
    /// (either analysis succeeded or an error occurred)
    /// </summary>
    Core::EventSignal<const ImageAnalysisEventArgs&>& Analyzed;

    /// <summary>
    /// Gets a collection of advanced Image Analysis properties.
    /// </summary>
    ImageAnalyzerProperties& Properties;

protected:
    /*! \cond PROTECTED */
    explicit ImageAnalyzer(AZAC_HANDLE view) :
        RecognizerBase(view),
        Analyzed(m_Recognized),
        Properties(GetProperties())
    {
    }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(ImageAnalyzer);
};

} } } } // Azure::AI::Vision::ImageAnalysis

